from typing import overload
from enum import IntEnum
import typing

import Common.Util
import QuantConnect
import System.Collections.Generic

Common_Util_ReadOnlyExtendedDictionary_TValue = typing.TypeVar("Common_Util_ReadOnlyExtendedDictionary_TValue")
Common_Util_ReadOnlyExtendedDictionary_TKey = typing.TypeVar("Common_Util_ReadOnlyExtendedDictionary_TKey")
Common_Util_BaseExtendedDictionary_TDictionary = typing.TypeVar("Common_Util_BaseExtendedDictionary_TDictionary")
Common_Util_BaseExtendedDictionary_TKey = typing.TypeVar("Common_Util_BaseExtendedDictionary_TKey")
Common_Util_BaseExtendedDictionary_TValue = typing.TypeVar("Common_Util_BaseExtendedDictionary_TValue")


class ReadOnlyExtendedDictionary(typing.Generic[Common_Util_ReadOnlyExtendedDictionary_TKey, Common_Util_ReadOnlyExtendedDictionary_TValue], Common.Util.BaseExtendedDictionary[Common_Util_ReadOnlyExtendedDictionary_TKey, Common_Util_ReadOnlyExtendedDictionary_TValue], System.Collections.Generic.IReadOnlyDictionary[Common_Util_ReadOnlyExtendedDictionary_TKey, Common_Util_ReadOnlyExtendedDictionary_TValue]):
    """This class has no documentation."""

    @property
    def is_read_only(self) -> bool:
        ...

    def __getitem__(self, key: Common_Util_ReadOnlyExtendedDictionary_TKey) -> Common_Util_ReadOnlyExtendedDictionary_TValue:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, dictionary: System.Collections.Generic.IDictionary[Common_Util_ReadOnlyExtendedDictionary_TKey, Common_Util_ReadOnlyExtendedDictionary_TValue]) -> None:
        ...

    @overload
    def __init__(self, data: System.Collections.Generic.IEnumerable[Common_Util_ReadOnlyExtendedDictionary_TValue], key_selector: typing.Callable[[Common_Util_ReadOnlyExtendedDictionary_TValue], Common_Util_ReadOnlyExtendedDictionary_TKey]) -> None:
        ...

    def __setitem__(self, key: Common_Util_ReadOnlyExtendedDictionary_TKey, value: Common_Util_ReadOnlyExtendedDictionary_TValue) -> None:
        ...

    @overload
    def add(self, key: Common_Util_ReadOnlyExtendedDictionary_TKey, value: Common_Util_ReadOnlyExtendedDictionary_TValue) -> None:
        ...

    @overload
    def add(self, item: System.Collections.Generic.KeyValuePair[Common_Util_ReadOnlyExtendedDictionary_TKey, Common_Util_ReadOnlyExtendedDictionary_TValue]) -> None:
        ...

    def clear(self) -> None:
        ...

    @overload
    def remove(self, key: Common_Util_ReadOnlyExtendedDictionary_TKey) -> bool:
        ...

    @overload
    def remove(self, item: System.Collections.Generic.KeyValuePair[Common_Util_ReadOnlyExtendedDictionary_TKey, Common_Util_ReadOnlyExtendedDictionary_TValue]) -> bool:
        ...


class BaseExtendedDictionary(typing.Generic[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue, Common_Util_BaseExtendedDictionary_TDictionary], QuantConnect.ExtendedDictionary[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue], System.Collections.Generic.IDictionary[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue], typing.Iterable[System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]]):
    """This class has no documentation."""

    @property
    def dictionary(self) -> Common_Util_BaseExtendedDictionary_TDictionary:
        ...

    @property
    def count(self) -> int:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def get_keys(self) -> typing.Iterable[Common_Util_BaseExtendedDictionary_TKey]:
        ...

    @property
    def get_values(self) -> typing.Iterable[Common_Util_BaseExtendedDictionary_TValue]:
        ...

    def __contains__(self, key: Common_Util_BaseExtendedDictionary_TKey) -> bool:
        ...

    def __getitem__(self, key: Common_Util_BaseExtendedDictionary_TKey) -> Common_Util_BaseExtendedDictionary_TValue:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, dictionary: Common_Util_BaseExtendedDictionary_TDictionary) -> None:
        ...

    @overload
    def __init__(self, data: System.Collections.Generic.IEnumerable[Common_Util_BaseExtendedDictionary_TValue], key_selector: typing.Callable[[Common_Util_BaseExtendedDictionary_TValue], Common_Util_BaseExtendedDictionary_TKey]) -> None:
        ...

    @overload
    def __init__(self, dictionary: System.Collections.Generic.IDictionary[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]) -> None:
        ...

    def __iter__(self) -> typing.Iterator[System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]]:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, key: Common_Util_BaseExtendedDictionary_TKey, value: Common_Util_BaseExtendedDictionary_TValue) -> None:
        ...

    @overload
    def add(self, key: Common_Util_BaseExtendedDictionary_TKey, value: Common_Util_BaseExtendedDictionary_TValue) -> None:
        ...

    @overload
    def add(self, item: System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]) -> None:
        ...

    def clear(self) -> None:
        ...

    def contains(self, item: System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]) -> bool:
        ...

    def contains_key(self, key: Common_Util_BaseExtendedDictionary_TKey) -> bool:
        ...

    def copy_to(self, array: typing.List[System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]], array_index: int) -> None:
        ...

    def get_enumerator(self) -> System.Collections.Generic.IEnumerator[System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]]:
        ...

    def get_items(self) -> System.Collections.Generic.IEnumerable[System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]]:
        ...

    @overload
    def remove(self, key: Common_Util_BaseExtendedDictionary_TKey) -> bool:
        ...

    @overload
    def remove(self, item: System.Collections.Generic.KeyValuePair[Common_Util_BaseExtendedDictionary_TKey, Common_Util_BaseExtendedDictionary_TValue]) -> bool:
        ...

    def try_get_value(self, key: Common_Util_BaseExtendedDictionary_TKey, value: typing.Optional[Common_Util_BaseExtendedDictionary_TValue]) -> typing.Tuple[bool, Common_Util_BaseExtendedDictionary_TValue]:
        ...





from typing import overload
from enum import IntEnum
import abc
import typing

import QuantConnect
import QuantConnect.Commands
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.IO

DynamicObject = typing.Any


class ICommand(metaclass=abc.ABCMeta):
    """Represents a command that can be run against a single algorithm"""

    @property
    @abc.abstractmethod
    def id(self) -> str:
        """Unique command id"""
        ...

    @id.setter
    def id(self, value: str) -> None:
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Runs this command against the specified algorithm instance
        
        :param algorithm: The algorithm to run this command against
        """
        ...


class CommandResultPacket(QuantConnect.Packets.Packet):
    """Contains data held as the result of executing a command"""

    @property
    def command_name(self) -> str:
        """Gets or sets the command that produced this packet"""
        ...

    @command_name.setter
    def command_name(self, value: str) -> None:
        ...

    @property
    def success(self) -> typing.Optional[bool]:
        """Gets or sets whether or not the"""
        ...

    @success.setter
    def success(self, value: typing.Optional[bool]) -> None:
        ...

    def __init__(self, command: QuantConnect.Commands.ICommand, success: typing.Optional[bool]) -> None:
        """Initializes a new instance of the CommandResultPacket class"""
        ...


class BaseCommand(System.Object, QuantConnect.Commands.ICommand, metaclass=abc.ABCMeta):
    """Base command implementation"""

    @property
    def id(self) -> str:
        """Unique command id"""
        ...

    @id.setter
    def id(self, value: str) -> None:
        ...

    def get_symbol(self, ticker: str, security_type: QuantConnect.SecurityType, market: str, symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract, QuantConnect.Securities.Security] = None) -> QuantConnect.Symbol:
        """
        Creats symbol using symbol properties.
        
        
        This codeEntityType is protected.
        
        :param ticker: The string ticker symbol
        :param security_type: The security type of the ticker. If security_type == Option, then a canonical symbol is created
        :param market: The market the ticker resides in
        :param symbol: The algorithm to run this command against
        """
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Runs this command against the specified algorithm instance
        
        :param algorithm: The algorithm to run this command against
        """
        ...


class AddSecurityCommand(QuantConnect.Commands.BaseCommand):
    """Represents a command to add a security to the algorithm"""

    class Result(QuantConnect.Commands.CommandResultPacket):
        """Result packet type for the AddSecurityCommand command"""

        @property
        def symbol(self) -> QuantConnect.Symbol:
            """The symbol result from the add security command"""
            ...

        @symbol.setter
        def symbol(self, value: QuantConnect.Symbol) -> None:
            ...

        def __init__(self, command: QuantConnect.Commands.AddSecurityCommand, success: bool, symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract, QuantConnect.Securities.Security]) -> None:
            """Initializes a new instance of the Result class"""
            ...

    @property
    def security_type(self) -> QuantConnect.SecurityType:
        """The security type of the security"""
        ...

    @security_type.setter
    def security_type(self, value: QuantConnect.SecurityType) -> None:
        ...

    @property
    def symbol(self) -> str:
        """The security's ticker symbol"""
        ...

    @symbol.setter
    def symbol(self, value: str) -> None:
        ...

    @property
    def resolution(self) -> QuantConnect.Resolution:
        """The requested resolution, defaults to Resolution.Minute"""
        ...

    @resolution.setter
    def resolution(self, value: QuantConnect.Resolution) -> None:
        ...

    @property
    def market(self) -> str:
        """The security's market, defaults to QuantConnect.Market.USA except for Forex, defaults to QuantConnect.Market.FXCM"""
        ...

    @market.setter
    def market(self, value: str) -> None:
        ...

    @property
    def fill_data_forward(self) -> bool:
        """The fill forward behavior, true to fill forward, false otherwise - defaults to true"""
        ...

    @fill_data_forward.setter
    def fill_data_forward(self, value: bool) -> None:
        ...

    @property
    def leverage(self) -> float:
        """The leverage for the security, defaults to 2 for equity, 50 for forex, and 1 for everything else"""
        ...

    @leverage.setter
    def leverage(self, value: float) -> None:
        ...

    @property
    def extended_market_hours(self) -> bool:
        """The extended market hours flag, true to allow pre/post market data, false for only in market data"""
        ...

    @extended_market_hours.setter
    def extended_market_hours(self, value: bool) -> None:
        ...

    def __init__(self) -> None:
        """Default construct that applies default values"""
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Runs this command against the specified algorithm instance
        
        :param algorithm: The algorithm to run this command against
        """
        ...


class AlgorithmStatusCommand(QuantConnect.Commands.BaseCommand):
    """Represents a command that will change the algorithm's status"""

    @property
    def status(self) -> QuantConnect.AlgorithmStatus:
        """Gets or sets the algorithm status"""
        ...

    @status.setter
    def status(self, value: QuantConnect.AlgorithmStatus) -> None:
        ...

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of the AlgorithmStatusCommand"""
        ...

    @overload
    def __init__(self, status: QuantConnect.AlgorithmStatus) -> None:
        """
        Initializes a new instance of the AlgorithmStatusCommand with
        the specified status
        """
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Sets the algorithm's status to status
        
        :param algorithm: The algorithm to run this command against
        """
        ...


class CallbackCommand(QuantConnect.Commands.BaseCommand):
    """Algorithm callback command type"""

    @property
    def type(self) -> str:
        """The target command type to run, if empty or null will be the generic untyped command handler"""
        ...

    @type.setter
    def type(self, value: str) -> None:
        ...

    @property
    def payload(self) -> str:
        """The command payload"""
        ...

    @payload.setter
    def payload(self, value: str) -> None:
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Runs this command against the specified algorithm instance
        
        :param algorithm: The algorithm to run this command against
        """
        ...

    def to_string(self) -> str:
        """The command string representation"""
        ...


class OrderCommand(QuantConnect.Commands.BaseCommand):
    """Represents a command to submit an order to the algorithm"""

    @property
    def symbol(self) -> QuantConnect.Symbol:
        """Gets or sets the symbol to be ordered"""
        ...

    @symbol.setter
    def symbol(self, value: QuantConnect.Symbol) -> None:
        ...

    @property
    def ticker(self) -> str:
        """Gets or sets the string ticker symbol"""
        ...

    @ticker.setter
    def ticker(self, value: str) -> None:
        ...

    @property
    def security_type(self) -> QuantConnect.SecurityType:
        """Gets or sets the security type of the ticker."""
        ...

    @security_type.setter
    def security_type(self, value: QuantConnect.SecurityType) -> None:
        ...

    @property
    def market(self) -> str:
        """Gets or sets the market the ticker resides in"""
        ...

    @market.setter
    def market(self, value: str) -> None:
        ...

    @property
    def order_type(self) -> QuantConnect.Orders.OrderType:
        """Gets or sets the order type to be submted"""
        ...

    @order_type.setter
    def order_type(self, value: QuantConnect.Orders.OrderType) -> None:
        ...

    @property
    def quantity(self) -> float:
        """Gets or sets the number of units to be ordered (directional)"""
        ...

    @quantity.setter
    def quantity(self, value: float) -> None:
        ...

    @property
    def limit_price(self) -> float:
        """Gets or sets the limit price. Only applies to QuantConnect.Orders.OrderType.Limit and QuantConnect.Orders.OrderType.StopLimit"""
        ...

    @limit_price.setter
    def limit_price(self, value: float) -> None:
        ...

    @property
    def stop_price(self) -> float:
        """Gets or sets the stop price. Only applies to QuantConnect.Orders.OrderType.StopLimit and QuantConnect.Orders.OrderType.StopMarket"""
        ...

    @stop_price.setter
    def stop_price(self, value: float) -> None:
        ...

    @property
    def tag(self) -> str:
        """Gets or sets an arbitrary tag to be attached to the order"""
        ...

    @tag.setter
    def tag(self, value: str) -> None:
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Runs this command against the specified algorithm instance
        
        :param algorithm: The algorithm to run this command against
        """
        ...

    def to_string(self) -> str:
        """
        Returns a string that represents the current object.
        
        :returns: A string that represents the current object.
        """
        ...


class QuitCommand(QuantConnect.Commands.AlgorithmStatusCommand):
    """Represents a command that will terminate the algorithm"""

    def __init__(self) -> None:
        """Initializes a new instance of the QuitCommand"""
        ...


class CancelOrderCommand(QuantConnect.Commands.BaseCommand):
    """Represents a command to cancel a specific order by id"""

    class Result(QuantConnect.Commands.CommandResultPacket):
        """Result packet type for the CancelOrderCommand command"""

        @property
        def quantity_filled(self) -> float:
            """Gets or sets the quantity filled on the cancelled order"""
            ...

        @quantity_filled.setter
        def quantity_filled(self, value: float) -> None:
            ...

        def __init__(self, command: QuantConnect.Commands.ICommand, success: bool, quantity_filled: float) -> None:
            """Initializes a new instance of the Result class"""
            ...

    @property
    def order_id(self) -> int:
        """Gets or sets the order id to be cancelled"""
        ...

    @order_id.setter
    def order_id(self, value: int) -> None:
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Runs this command against the specified algorithm instance
        
        :param algorithm: The algorithm to run this command against
        """
        ...


class ICommandHandler(System.IDisposable, metaclass=abc.ABCMeta):
    """
    Represents a command queue for the algorithm. This is an entry point
    for external messages to act upon the running algorithm instance.
    """

    def initialize(self, job: QuantConnect.Packets.AlgorithmNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        Initializes this command queue for the specified job
        
        :param job: The job that defines what queue to bind to
        :param algorithm: The algorithm instance
        """
        ...

    def process_commands(self) -> typing.Iterable[QuantConnect.Commands.CommandResultPacket]:
        """
        Process any commands in the queue
        
        :returns: The command result packet of each command executed if any.
        """
        ...


class BaseCommandHandler(System.Object, QuantConnect.Commands.ICommandHandler, metaclass=abc.ABCMeta):
    """Base algorithm command handler"""

    SETTINGS: typing.Any = ...
    """
    Command json settings
    
    
    This codeEntityType is protected.
    """

    @property
    def algorithm(self) -> QuantConnect.Interfaces.IAlgorithm:
        """
        The algorithm instance
        
        
        This codeEntityType is protected.
        """
        ...

    @algorithm.setter
    def algorithm(self, value: QuantConnect.Interfaces.IAlgorithm) -> None:
        ...

    def acknowledge(self, command: QuantConnect.Commands.ICommand, command_result_packet: QuantConnect.Commands.CommandResultPacket) -> None:
        """
        Acknowledge a command that has been executed
        
        
        This codeEntityType is protected.
        
        :param command: The command that was executed
        :param command_result_packet: The result
        """
        ...

    def dispose(self) -> None:
        """Disposes of this instance"""
        ...

    def get_commands(self) -> typing.Iterable[QuantConnect.Commands.ICommand]:
        """
        Get the commands to run
        
        
        This codeEntityType is protected.
        """
        ...

    def initialize(self, job: QuantConnect.Packets.AlgorithmNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        Initializes this command queue for the specified job
        
        :param job: The job that defines what queue to bind to
        :param algorithm: The algorithm instance
        """
        ...

    def process_commands(self) -> typing.Iterable[QuantConnect.Commands.CommandResultPacket]:
        """Will consumer and execute any command in the queue"""
        ...

    def try_get_callback_command(self, payload: str) -> QuantConnect.Commands.ICommand:
        """
        Helper method to create a callback command
        
        
        This codeEntityType is protected.
        """
        ...


class FileCommandHandler(QuantConnect.Commands.BaseCommandHandler):
    """Represents a command handler that sources it's commands from a file on the local disk"""

    def __init__(self) -> None:
        """
        Initializes a new instance of the FileCommandHandler class
        using the 'command-json-file' configuration value for the command json file
        """
        ...

    def acknowledge(self, command: QuantConnect.Commands.ICommand, command_result_packet: QuantConnect.Commands.CommandResultPacket) -> None:
        """
        Acknowledge a command that has been executed
        
        
        This codeEntityType is protected.
        
        :param command: The command that was executed
        :param command_result_packet: The result
        """
        ...

    @staticmethod
    def get_command_files() -> typing.Iterable[System.IO.FileInfo]:
        """
        Gets all the available command files
        
        :returns: Sorted enumerator of all the available command files.
        """
        ...

    def get_commands(self) -> typing.Iterable[QuantConnect.Commands.ICommand]:
        """
        Gets the next command in the queue
        
        
        This codeEntityType is protected.
        
        :returns: The next command in the queue, if present, null if no commands present.
        """
        ...


class LiquidateCommand(QuantConnect.Commands.BaseCommand):
    """Represents a command that will liquidate the entire algorithm"""

    @property
    def ticker(self) -> str:
        """Gets or sets the string ticker symbol"""
        ...

    @ticker.setter
    def ticker(self, value: str) -> None:
        ...

    @property
    def security_type(self) -> QuantConnect.SecurityType:
        """Gets or sets the security type of the ticker."""
        ...

    @security_type.setter
    def security_type(self, value: QuantConnect.SecurityType) -> None:
        ...

    @property
    def market(self) -> str:
        """Gets or sets the market the ticker resides in"""
        ...

    @market.setter
    def market(self, value: str) -> None:
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Submits orders to liquidate all current holdings in the algorithm
        
        :param algorithm: The algorithm to be liquidated
        """
        ...


class UpdateOrderCommand(QuantConnect.Commands.BaseCommand):
    """Represents a command to update an order by id"""

    @property
    def order_id(self) -> int:
        """Gets or sets the id of the order to update"""
        ...

    @order_id.setter
    def order_id(self, value: int) -> None:
        ...

    @property
    def quantity(self) -> typing.Optional[float]:
        """Gets or sets the new quantity, specify null to not update the quantity"""
        ...

    @quantity.setter
    def quantity(self, value: typing.Optional[float]) -> None:
        ...

    @property
    def limit_price(self) -> typing.Optional[float]:
        """
        Gets or sets the new limit price, specify null to not update the limit price.
        This will only be used if the order has a limit price (Limit/StopLimit orders)
        """
        ...

    @limit_price.setter
    def limit_price(self, value: typing.Optional[float]) -> None:
        ...

    @property
    def stop_price(self) -> typing.Optional[float]:
        """
        Gets or sets the new stop price, specify null to not update the stop price.
        This will onky be used if the order has a stop price (StopLimit/StopMarket orders)
        """
        ...

    @stop_price.setter
    def stop_price(self, value: typing.Optional[float]) -> None:
        ...

    @property
    def tag(self) -> str:
        """Gets or sets the new tag for the order, specify null to not update the tag"""
        ...

    @tag.setter
    def tag(self, value: str) -> None:
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Commands.CommandResultPacket:
        """
        Runs this command against the specified algorithm instance
        
        :param algorithm: The algorithm to run this command against
        """
        ...


class Command(DynamicObject):
    """Base generic dynamic command class"""

    @property
    def payload_data(self) -> str:
        """
        Useful to string representation in python
        
        
        This codeEntityType is protected.
        """
        ...

    @payload_data.setter
    def payload_data(self, value: str) -> None:
        ...

    def get_meta_object(self, parameter: typing.Any) -> typing.Any:
        """Get the metaObject required for Dynamism."""
        ...

    def get_property(self, name: str) -> System.Object:
        """
        Gets the property's value with the specified name. This is a case-insensitve search.
        
        :param name: The property name to access
        :returns: object value of BaseData.
        """
        ...

    def run(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> typing.Optional[bool]:
        """
        Run this command using the target algorithm
        
        :param algorithm: The algorithm instance
        :returns: True if success, false otherwise. Returning null will disable command feedback.
        """
        ...

    def set_property(self, name: str, value: typing.Any) -> System.Object:
        """
        Sets the property with the specified name to the value. This is a case-insensitve search.
        
        :param name: The property name to set
        :param value: The new property value
        :returns: Returns the input value back to the caller.
        """
        ...

    def to_string(self) -> str:
        """The string representation of this command"""
        ...





from typing import overload
from enum import IntEnum
import abc
import datetime
import typing

import QuantConnect
import QuantConnect.Data
import QuantConnect.Data.Consolidators
import QuantConnect.Data.Market
import QuantConnect.Python
import System

QuantConnect_Data_Consolidators_ClassicRenkoConsolidator = typing.Any
QuantConnect_Data_Consolidators_RenkoConsolidator = typing.Any

QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T = typing.TypeVar("QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T")
QuantConnect_Data_Consolidators_FilteredIdentityDataConsolidator_T = typing.TypeVar("QuantConnect_Data_Consolidators_FilteredIdentityDataConsolidator_T")
QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated = typing.TypeVar("QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated")
QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_T = typing.TypeVar("QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_T")
QuantConnect_Data_Consolidators_IdentityDataConsolidator_T = typing.TypeVar("QuantConnect_Data_Consolidators_IdentityDataConsolidator_T")
QuantConnect_Data_Consolidators_ClassicRenkoConsolidator_TInput = typing.TypeVar("QuantConnect_Data_Consolidators_ClassicRenkoConsolidator_TInput")
QuantConnect_Data_Consolidators_TradeBarConsolidatorBase_T = typing.TypeVar("QuantConnect_Data_Consolidators_TradeBarConsolidatorBase_T")
QuantConnect_Data_Consolidators_RenkoConsolidator_TInput = typing.TypeVar("QuantConnect_Data_Consolidators_RenkoConsolidator_TInput")
QuantConnect_Data_Consolidators_WickedRenkoConsolidator_T = typing.TypeVar("QuantConnect_Data_Consolidators_WickedRenkoConsolidator_T")
QuantConnect_Data_Consolidators_DataConsolidator_TInput = typing.TypeVar("QuantConnect_Data_Consolidators_DataConsolidator_TInput")
QuantConnect_Data_Consolidators__EventContainer_Callable = typing.TypeVar("QuantConnect_Data_Consolidators__EventContainer_Callable")
QuantConnect_Data_Consolidators__EventContainer_ReturnType = typing.TypeVar("QuantConnect_Data_Consolidators__EventContainer_ReturnType")


class RangeConsolidator(QuantConnect.Data.Consolidators.BaseTimelessConsolidator[QuantConnect.Data.Market.RangeBar]):
    """This consolidator can transform a stream of IBaseData instances into a stream of RangeBar"""

    @property
    def current_bar(self) -> QuantConnect.Data.Market.RangeBar:
        """
        Bar being created
        
        
        This codeEntityType is protected.
        """
        ...

    @current_bar.setter
    def current_bar(self, value: QuantConnect.Data.Market.RangeBar) -> None:
        ...

    @property
    def range_size(self) -> float:
        """
        Range for each RangeBar, this is, the difference between the High and Low for each
        RangeBar
        """
        ...

    @property
    def range(self) -> int:
        """Number of MinimumPriceVariation units"""
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets RangeBar which is the type emitted in the IDataConsolidator.data_consolidated event."""
        ...

    @property
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @overload
    def __init__(self, range: int, selector: typing.Any, volume_selector: typing.Any = None) -> None:
        """
        Initializes a new instance of the RangeConsolidator class.
        
        :param range: The Range interval sets the range in which the price moves, which in turn initiates the formation of a new bar.
        One range equals to one minimum price change, where this last value is defined depending of the RangeBar's symbol
        :param selector: Extracts the value from a data instance to be formed into a RangeBar. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        """
        ...

    @overload
    def __init__(self, range: int, selector: typing.Callable[[QuantConnect.Data.IBaseData], float] = None, volume_selector: typing.Callable[[QuantConnect.Data.IBaseData], float] = None) -> None:
        """
        Initializes a new instance of the RangeConsolidator class.
        
        :param range: The Range interval sets the range in which the price moves, which in turn initiates the formation of a new bar.
        One range equals to one minimum price change, where this last value is defined depending of the RangeBar's symbol
        :param selector: Extracts the value from a data instance to be formed into a RangeBar. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar, except if the input is a TradeBar.
        """
        ...

    def create_new_bar(self, data: QuantConnect.Data.IBaseData, current_value: float, volume: float) -> None:
        """
        Creates a new bar with the given data
        
        
        This codeEntityType is protected.
        
        :param data: The new data for the bar
        :param current_value: The new value for the bar
        :param volume: The new volume for the bar
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def update_bar(self, time: typing.Union[datetime.datetime, datetime.date], current_value: float, volume: float) -> None:
        """
        Updates the current RangeBar being created with the given data.
        Additionally, if it's the case, it consolidates the current RangeBar
        
        
        This codeEntityType is protected.
        
        :param time: Time of the given data
        :param current_value: Value of the given data
        :param volume: Volume of the given data
        """
        ...


class ClassicRangeConsolidator(QuantConnect.Data.Consolidators.RangeConsolidator):
    """
    This consolidator can transform a stream of IBaseData instances into a stream of RangeBar.
    The difference between this consolidator and RangeConsolidator, is that this last one creates intermediate/
    phantom RangeBar's (RangeBar's with zero volume) if the price rises up or falls down by above/below two times the range
    size. Therefore, RangeConsolidator leaves no space between two adyacent RangeBar's since it always start
    a new RangeBar one range above the last RangeBar's High value or one range below the last RangeBar's Low value, where
    one range equals to one minimum price change.
    """

    @overload
    def __init__(self, range: int, selector: typing.Any, volume_selector: typing.Any = None) -> None:
        """
        Initializes a new instance of the RangeConsolidator class.
        
        :param range: The Range interval sets the range in which the price moves, which in turn initiates the formation of a new bar.
        One range equals to one minimum price change, where this last value is defined depending of the RangeBar's symbol
        :param selector: Extracts the value from a data instance to be formed into a RangeBar. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        """
        ...

    @overload
    def __init__(self, range: int, selector: typing.Callable[[QuantConnect.Data.IBaseData], float] = None, volume_selector: typing.Callable[[QuantConnect.Data.IBaseData], float] = None) -> None:
        """
        Initializes a new instance of the ClassicRangeConsolidator class.
        
        :param range: The Range interval sets the range in which the price moves, which in turn initiates the formation of a new bar.
        One range equals to one minimum price change, where this last value is defined depending of the RangeBar's symbol
        :param selector: Extracts the value from a data instance to be formed into a RangeBar. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar, except if the input is a TradeBar.
        """
        ...

    def update_bar(self, time: typing.Union[datetime.datetime, datetime.date], current_value: float, volume: float) -> None:
        """
        Updates the current RangeBar being created with the given data.
        Additionally, if it's the case, it consolidates the current RangeBar
        
        
        This codeEntityType is protected.
        
        :param time: Time of the given data
        :param current_value: Value of the given data
        :param volume: Volume of the given data
        """
        ...


class IDataConsolidator(System.IDisposable, metaclass=abc.ABCMeta):
    """
    Represents a type capable of taking BaseData updates and firing events containing new
    'consolidated' data. These types can be used to produce larger bars, or even be used to
    transform the data before being sent to another component. The most common usage of these
    types is with indicators.
    """

    @property
    @abc.abstractmethod
    def consolidated(self) -> QuantConnect.Data.IBaseData:
        """
        Gets the most recently consolidated piece of data. This will be null if this consolidator
        has not produced any data yet.
        """
        ...

    @property
    @abc.abstractmethod
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    @abc.abstractmethod
    def input_type(self) -> typing.Type:
        """Gets the type consumed by this consolidator"""
        ...

    @property
    @abc.abstractmethod
    def output_type(self) -> typing.Type:
        """Gets the type produced by this consolidator"""
        ...

    @property
    @abc.abstractmethod
    def data_consolidated(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any], typing.Any]:
        """Event handler that fires when a new piece of data is produced"""
        ...

    @data_consolidated.setter
    def data_consolidated(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any], typing.Any]) -> None:
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    def update(self, data: QuantConnect.Data.IBaseData) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...


class BaseTimelessConsolidator(typing.Generic[QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T], System.Object, QuantConnect.Data.Consolidators.IDataConsolidator, metaclass=abc.ABCMeta):
    """
    Represents a timeless consolidator which depends on the given values. This consolidator
    is meant to consolidate data into bars that do not depend on time, e.g., RangeBar's.
    """

    @property
    def selector(self) -> typing.Callable[[QuantConnect.Data.IBaseData], float]:
        """
        Extracts the value from a data instance to be formed into a T.
        
        
        This codeEntityType is protected.
        """
        ...

    @selector.setter
    def selector(self, value: typing.Callable[[QuantConnect.Data.IBaseData], float]) -> None:
        ...

    @property
    def volume_selector(self) -> typing.Callable[[QuantConnect.Data.IBaseData], float]:
        """
        Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        
        
        This codeEntityType is protected.
        """
        ...

    @volume_selector.setter
    def volume_selector(self, value: typing.Callable[[QuantConnect.Data.IBaseData], float]) -> None:
        ...

    @property
    def data_consolidated_handler(self) -> typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any]:
        """
        Event handler type for the IDataConsolidator.DataConsolidated event
        
        
        This codeEntityType is protected.
        """
        ...

    @data_consolidated_handler.setter
    def data_consolidated_handler(self, value: typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any]) -> None:
        ...

    @property
    def current_bar(self) -> QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T:
        """
        Bar being created
        
        
        This codeEntityType is protected.
        """
        ...

    @current_bar.setter
    def current_bar(self, value: QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T) -> None:
        ...

    @property
    def consolidated(self) -> QuantConnect.Data.IBaseData:
        """
        Gets the most recently consolidated piece of data. This will be null if this consolidator
        has not produced any data yet.
        """
        ...

    @consolidated.setter
    def consolidated(self, value: QuantConnect.Data.IBaseData) -> None:
        ...

    @property
    @abc.abstractmethod
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def input_type(self) -> typing.Type:
        """Gets the type consumed by this consolidator"""
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets T which is the type emitted in the IDataConsolidator.data_consolidated event."""
        ...

    @property
    def data_consolidated(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T], typing.Any], typing.Any]:
        """Event handler that fires when a new piece of data is produced"""
        ...

    @data_consolidated.setter
    def data_consolidated(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T], typing.Any], typing.Any]) -> None:
        ...

    @overload
    def __init__(self, value_selector: typing.Any, volume_selector: typing.Any = None) -> None:
        """
        Initializes a new instance of the BaseTimelessConsolidator{T} class.
        
        
        This codeEntityType is protected.
        
        :param value_selector: Extracts the value from a data instance to be formed into a new bar which inherits from IBaseData. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        """
        ...

    @overload
    def __init__(self, selector: typing.Callable[[QuantConnect.Data.IBaseData], float] = None, volume_selector: typing.Callable[[QuantConnect.Data.IBaseData], float] = None) -> None:
        """
        Initializes a new instance of the BaseTimelessConsolidator{T} class.
        
        
        This codeEntityType is protected.
        
        :param selector: Extracts the value from a data instance to be formed into a new bar which inherits from IBaseData. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        """
        ...

    def create_new_bar(self, data: QuantConnect.Data.IBaseData, current_value: float, volume: float) -> None:
        """
        Creates a new bar with the given data
        
        
        This codeEntityType is protected.
        
        :param data: The new data for the bar
        :param current_value: The new value for the bar
        :param volume: The new volume to the bar
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def on_data_consolidated(self, consolidated: QuantConnect_Data_Consolidators_BaseTimelessConsolidator_T) -> None:
        """
        Event invocator for the DataConsolidated event. This should be invoked
        by derived classes when they have consolidated a new piece of data.
        
        
        This codeEntityType is protected.
        
        :param consolidated: The newly consolidated data
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    def update(self, data: QuantConnect.Data.IBaseData) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...

    def update_bar(self, time: typing.Union[datetime.datetime, datetime.date], current_value: float, volume: float) -> None:
        """
        Updates the current RangeBar being created with the given data.
        Additionally, if it's the case, it consolidates the current RangeBar
        
        
        This codeEntityType is protected.
        
        :param time: Time of the given data
        :param current_value: Value of the given data
        :param volume: Volume of the given data
        """
        ...


class SequentialConsolidator(System.Object, QuantConnect.Data.Consolidators.IDataConsolidator):
    """
    This consolidator wires up the events on its First and Second consolidators
    such that data flows from the First to Second consolidator. It's output comes
    from the Second.
    """

    @property
    def first(self) -> QuantConnect.Data.Consolidators.IDataConsolidator:
        """Gets the first consolidator to receive data"""
        ...

    @property
    def second(self) -> QuantConnect.Data.Consolidators.IDataConsolidator:
        """
        Gets the second consolidator that ends up receiving data produced
        by the first
        """
        ...

    @property
    def consolidated(self) -> QuantConnect.Data.IBaseData:
        """
        Gets the most recently consolidated piece of data. This will be null if this consolidator
        has not produced any data yet.
        
        For a SequentialConsolidator, this is the output from the 'Second' consolidator.
        """
        ...

    @property
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def input_type(self) -> typing.Type:
        """Gets the type consumed by this consolidator"""
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets the type produced by this consolidator"""
        ...

    @property
    def data_consolidated(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any], typing.Any]:
        """Event handler that fires when a new piece of data is produced"""
        ...

    @data_consolidated.setter
    def data_consolidated(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any], typing.Any]) -> None:
        ...

    def __init__(self, first: typing.Union[QuantConnect.Data.Consolidators.IDataConsolidator, QuantConnect.Python.PythonConsolidator, datetime.timedelta], second: typing.Union[QuantConnect.Data.Consolidators.IDataConsolidator, QuantConnect.Python.PythonConsolidator, datetime.timedelta]) -> None:
        """
        Creates a new consolidator that will pump date through the first, and then the output
        of the first into the second. This enables 'wrapping' or 'composing' of consolidators
        
        :param first: The first consolidator to receive data
        :param second: The consolidator to receive first's output
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def on_data_consolidated(self, consolidated: QuantConnect.Data.IBaseData) -> None:
        """
        Event invocator for the DataConsolidated event. This should be invoked
        by derived classes when they have consolidated a new piece of data.
        
        
        This codeEntityType is protected.
        
        :param consolidated: The newly consolidated data
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    def update(self, data: QuantConnect.Data.IBaseData) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...


class CalendarInfo:
    """Calendar Info for storing information related to the start and period of a consolidator"""

    @property
    def start(self) -> datetime.datetime:
        """Calendar Start"""
        ...

    @property
    def period(self) -> datetime.timedelta:
        """Consolidation Period"""
        ...

    @property
    def end(self) -> datetime.datetime:
        """Calendar End"""
        ...

    def __eq__(self, right: QuantConnect.Data.Consolidators.CalendarInfo) -> bool:
        """
        Indicates whether the given object is equal to this object, this is, the Calendar start
        and consolidation period is the same for both
        """
        ...

    def __init__(self, start: typing.Union[datetime.datetime, datetime.date], period: datetime.timedelta) -> None:
        """
        Constructor for CalendarInfo; used for consolidation calendar
        
        :param start: Calendar Start
        :param period: Consolidation Period
        """
        ...

    def __ne__(self, right: QuantConnect.Data.Consolidators.CalendarInfo) -> bool:
        """
        Indicates whether the given object is equal to this object, this is, the Calendar start
        and consolidation period is the same for both
        """
        ...

    def equals(self, obj: typing.Any) -> bool:
        """
        Indicates whether the given object is equal to this object, this is, the Calendar start
        and consolidation period is the same for both
        """
        ...

    def get_hash_code(self) -> int:
        """Returns the hash code for this object as an integer"""
        ...

    def to_string(self) -> str:
        """Returns a string containing the Calendar start and the consolidation period"""
        ...


class CalendarType(System.Object):
    """
    Calendar Type Class; now obsolete routes functions to Calendar
    
    CalendarType is obsolete, please use Calendar instead
    """

    WEEKLY: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]
    """Computes the start of week (previous Monday) of given date/time"""

    MONTHLY: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]
    """Computes the start of month (1st of the current month) of given date/time"""


class FilteredIdentityDataConsolidator(typing.Generic[QuantConnect_Data_Consolidators_FilteredIdentityDataConsolidator_T], QuantConnect.Data.Consolidators.IdentityDataConsolidator[QuantConnect_Data_Consolidators_FilteredIdentityDataConsolidator_T]):
    """
    Provides an implementation of IDataConsolidator that preserve the input
    data unmodified. The input data is filtering by the specified predicate function
    """

    def __init__(self, predicate: typing.Callable[[QuantConnect_Data_Consolidators_FilteredIdentityDataConsolidator_T], bool]) -> None:
        """
        Initializes a new instance of the FilteredIdentityDataConsolidator{T} class
        
        :param predicate: The predicate function, returning true to accept data and false to reject data
        """
        ...

    @staticmethod
    def for_tick_type(tick_type: QuantConnect.TickType) -> QuantConnect.Data.Consolidators.FilteredIdentityDataConsolidator[QuantConnect.Data.Market.Tick]:
        """
        Creates a new instance of FilteredIdentityDataConsolidator{T} that filters ticks
        based on the specified TickType
        
        :param tick_type: The tick type of data to accept
        :returns: A new FilteredIdentityDataConsolidator{T} that filters based on the provided tick type.
        """
        ...

    def update(self, data: QuantConnect_Data_Consolidators_FilteredIdentityDataConsolidator_T) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...


class BaseDataConsolidator(QuantConnect.Data.Consolidators.TradeBarConsolidatorBase[QuantConnect.Data.BaseData]):
    """Type capable of consolidating trade bars from any base data instance"""

    @overload
    def __init__(self, pyfuncobj: typing.Any) -> None:
        """
        Initializes a new instance of the BaseDataConsolidator class
        
        :param pyfuncobj: Func that defines the start time of a consolidated data
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the period
        
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Initializes a new instance of the BaseDataConsolidator class
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.TradeBar, data: QuantConnect.Data.BaseData) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new trade bar
        :param data: The new data
        """
        ...

    @staticmethod
    def from_resolution(resolution: QuantConnect.Resolution) -> QuantConnect.Data.Consolidators.BaseDataConsolidator:
        """
        Create a new TickConsolidator for the desired resolution
        
        :param resolution: The resolution desired
        :returns: A consolidator that produces data on the resolution interval.
        """
        ...


class PeriodCountConsolidatorBase(typing.Generic[QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_T, QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated], QuantConnect.Data.Consolidators.DataConsolidator[QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_T], metaclass=abc.ABCMeta):
    """
    Provides a base class for consolidators that emit data based on the passing of a period of time
    or after seeing a max count of data points.
    """

    @property
    def _working_bar(self) -> QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated:
        """This codeEntityType is protected."""
        ...

    @_working_bar.setter
    def _working_bar(self, value: QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated) -> None:
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets the type produced by this consolidator"""
        ...

    @property
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def data_consolidated(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated], typing.Any], typing.Any]:
        """
        Event handler that fires when a new piece of data is produced. We define this as a 'new'
        event so we can expose it as a t_consolidated instead of a BaseData instance
        """
        ...

    @data_consolidated.setter
    def data_consolidated(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated], typing.Any], typing.Any]) -> None:
        ...

    @property
    def is_time_based(self) -> bool:
        """
        Returns true if this consolidator is time-based, false otherwise
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def period(self) -> typing.Optional[datetime.timedelta]:
        """
        Gets the time period for this consolidator
        
        
        This codeEntityType is protected.
        """
        ...

    @overload
    def __init__(self, py_object: typing.Any) -> None:
        """
        Creates a consolidator to produce a new t_consolidated instance representing the last count pieces of data or the period, whichever comes first
        
        
        This codeEntityType is protected.
        
        :param py_object: Python object that defines either a function object that defines the start time of a consolidated data or a timespan
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta, start_time: typing.Optional[datetime.timedelta] = None) -> None:
        """
        Creates a consolidator to produce a new t_consolidated instance representing the period
        
        
        This codeEntityType is protected.
        
        :param period: The minimum span of time before emitting a consolidated bar
        :param start_time: Optionally the bar start time anchor to use
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Creates a consolidator to produce a new t_consolidated instance representing the last count pieces of data
        
        
        This codeEntityType is protected.
        
        :param max_count: The number of pieces to accept before emiting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new t_consolidated instance representing the last count pieces of data or the period, whichever comes first
        
        
        This codeEntityType is protected.
        
        :param max_count: The number of pieces to accept before emiting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Creates a consolidator to produce a new t_consolidated instance representing the last count pieces of data or the period, whichever comes first
        
        
        This codeEntityType is protected.
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated, data: QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_T) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new consolidated bar
        :param data: The new data
        """
        ...

    @overload
    def get_rounded_bar_time(self, time: typing.Union[datetime.datetime, datetime.date]) -> datetime.datetime:
        """
        Gets a rounded-down bar time. Called by AggregateBar in derived classes.
        
        
        This codeEntityType is protected.
        
        :param time: The bar time to be rounded down
        :returns: The rounded bar time.
        """
        ...

    @overload
    def get_rounded_bar_time(self, input_data: QuantConnect.Data.IBaseData) -> datetime.datetime:
        """
        Gets a rounded-down bar start time. Called by AggregateBar in derived classes.
        
        
        This codeEntityType is protected.
        
        :param input_data: The input data point
        :returns: The rounded bar start time.
        """
        ...

    def on_data_consolidated(self, e: QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_TConsolidated) -> None:
        """
        Event invocator for the data_consolidated event
        
        
        This codeEntityType is protected.
        
        :param e: The consolidated data
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def reset_working_bar(self) -> None:
        """
        Resets the working bar
        
        
        This codeEntityType is protected.
        """
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    def should_process(self, data: QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_T) -> bool:
        """
        Determines whether or not the specified data should be processed
        
        
        This codeEntityType is protected.
        
        :param data: The data to check
        :returns: True if the consolidator should process this data, false otherwise.
        """
        ...

    def update(self, data: QuantConnect_Data_Consolidators_PeriodCountConsolidatorBase_T) -> None:
        """
        Updates this consolidator with the specified data. This method is
        responsible for raising the DataConsolidated event
        In time span mode, the bar range is closed on the left and open on the right: [T, T+TimeSpan).
        For example, if time span is 1 minute, we have [10:00, 10:01): so data at 10:01 is not
        included in the bar starting at 10:00.
        
        :param data: The new data for the consolidator
        """
        ...


class IdentityDataConsolidator(typing.Generic[QuantConnect_Data_Consolidators_IdentityDataConsolidator_T], QuantConnect.Data.Consolidators.DataConsolidator[QuantConnect_Data_Consolidators_IdentityDataConsolidator_T]):
    """
    Represents the simplest DataConsolidator implementation, one that is defined
    by a straight pass through of the data. No projection or aggregation is performed.
    """

    @property
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets the type produced by this consolidator"""
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    def update(self, data: QuantConnect_Data_Consolidators_IdentityDataConsolidator_T) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...


class Calendar(System.Object):
    """Helper class that provides Func{DateTime,CalendarInfo} used to define consolidation calendar"""

    WEEKLY: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]
    """Computes the start of week (previous Monday) of given date/time"""

    MONTHLY: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]
    """Computes the start of month (1st of the current month) of given date/time"""

    QUARTERLY: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]
    """Computes the start of quarter (1st of the starting month of current quarter) of given date/time"""

    YEARLY: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]
    """Computes the start of year (1st of the current year) of given date/time"""


class VolumeRenkoConsolidator(QuantConnect.Data.Consolidators.DataConsolidator[QuantConnect.Data.BaseData]):
    """
    This consolidator can transform a stream of BaseData instances into a stream of RenkoBar
    with a constant volume for each bar.
    """

    @property
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets VolumeRenkoBar which is the type emitted in the IDataConsolidator.data_consolidated event."""
        ...

    @property
    def data_consolidated(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.Market.VolumeRenkoBar], typing.Any], typing.Any]:
        """Event handler that fires when a new piece of data is produced"""
        ...

    @data_consolidated.setter
    def data_consolidated(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.Market.VolumeRenkoBar], typing.Any], typing.Any]) -> None:
        ...

    def __init__(self, bar_size: float) -> None:
        """
        Initializes a new instance of the VolumeRenkoConsolidator class using the specified bar_size.
        
        :param bar_size: The constant volume size of each bar
        """
        ...

    def adjust_volume(self, volume: float, price: float) -> float:
        """
        Returns the raw volume without any adjustment.
        
        
        This codeEntityType is protected.
        
        :param volume: The volume
        :param price: The price
        :returns: The unmodified volume.
        """
        ...

    def on_data_consolidated(self, consolidated: QuantConnect.Data.Market.VolumeRenkoBar) -> None:
        """
        Event invocator for the DataConsolidated event. This should be invoked
        by derived classes when they have consolidated a new piece of data.
        
        
        This codeEntityType is protected.
        
        :param consolidated: The newly consolidated data
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    def update(self, data: QuantConnect.Data.BaseData) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...


class TradeBarConsolidator(QuantConnect.Data.Consolidators.TradeBarConsolidatorBase[QuantConnect.Data.Market.TradeBar]):
    """
    A data consolidator that can make bigger bars from smaller ones over a given
    time span or a count of pieces of data.
    
    Use this consolidator to turn data of a lower resolution into data of a higher resolution,
    for example, if you subscribe to minute data but want to have a 15 minute bar.
    """

    @overload
    def __init__(self, pyfuncobj: typing.Any) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        :param pyfuncobj: Python function object that defines the start time of a consolidated data
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta, start_time: typing.Optional[datetime.timedelta] = None) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the period
        
        :param period: The minimum span of time before emitting a consolidated bar
        :param start_time: Optionally the bar start time anchor to use
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.TradeBar, data: QuantConnect.Data.Market.TradeBar) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new trade bar
        :param data: The new data
        """
        ...

    @staticmethod
    def from_resolution(resolution: QuantConnect.Resolution) -> QuantConnect.Data.Consolidators.TradeBarConsolidator:
        """
        Create a new TradeBarConsolidator for the desired resolution
        
        :param resolution: The resolution desired
        :returns: A consolidator that produces data on the resolution interval.
        """
        ...


class ClassicRenkoConsolidator(typing.Generic[QuantConnect_Data_Consolidators_ClassicRenkoConsolidator_TInput], QuantConnect_Data_Consolidators_ClassicRenkoConsolidator):
    """Provides a type safe wrapper on the RenkoConsolidator class. This just allows us to define our selector functions with the real type they'll be receiving"""

    @property
    def current_bar(self) -> QuantConnect.Data.Market.RenkoBar:
        """
        Bar being created
        
        
        This codeEntityType is protected.
        """
        ...

    @current_bar.setter
    def current_bar(self, value: QuantConnect.Data.Market.RenkoBar) -> None:
        ...

    @property
    def type(self) -> QuantConnect.Data.Market.RenkoType:
        """Gets the kind of the bar"""
        ...

    @property
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets RenkoBar which is the type emitted in the IDataConsolidator.data_consolidated event."""
        ...

    @overload
    def __init__(self, bar_size: float, selector: typing.Any, volume_selector: typing.Any = None, even_bars: bool = True) -> None:
        """
        Initializes a new instance of the ClassicRenkoConsolidator class.
        
        :param bar_size: The size of each bar in units of the value produced by selector
        :param selector: Extracts the value from a data instance to be formed into a RenkoBar. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        :param even_bars: When true bar open/close will be a multiple of the bar_size
        """
        ...

    @overload
    def __init__(self, bar_size: float, even_bars: bool = True) -> None:
        """
        Initializes a new instance of the ClassicRenkoConsolidator class using the specified bar_size.
        The value selector will by default select IBaseData.value
        The volume selector will by default select zero.
        
        :param bar_size: The constant value size of each bar
        :param even_bars: When true bar open/close will be a multiple of the bar_size
        """
        ...

    @overload
    def __init__(self, bar_size: float, selector: typing.Callable[[QuantConnect.Data.IBaseData], float], volume_selector: typing.Callable[[QuantConnect.Data.IBaseData], float] = None, even_bars: bool = True) -> None:
        """
        Initializes a new instance of the ClassicRenkoConsolidator class.
        
        :param bar_size: The size of each bar in units of the value produced by selector
        :param selector: Extracts the value from a data instance to be formed into a RenkoBar. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        :param even_bars: When true bar open/close will be a multiple of the bar_size
        """
        ...

    @overload
    def __init__(self, bar_size: float, selector: typing.Callable[[QuantConnect_Data_Consolidators_ClassicRenkoConsolidator_TInput], float], volume_selector: typing.Callable[[QuantConnect_Data_Consolidators_ClassicRenkoConsolidator_TInput], float] = None, even_bars: bool = True) -> None:
        """
        Initializes a new instance of the ClassicRenkoConsolidator class.
        
        :param bar_size: The size of each bar in units of the value produced by selector
        :param selector: Extracts the value from a data instance to be formed into a RenkoBar. The default
        value is (x => x.Value) the IBaseData.value property on IBaseData
        :param volume_selector: Extracts the volume from a data instance. The default value is null which does
        not aggregate volume per bar.
        :param even_bars: When true bar open/close will be a multiple of the bar_size
        """
        ...

    @overload
    def __init__(self, bar_size: float, type: QuantConnect.Data.Market.RenkoType) -> None:
        """
        Initializes a new instance of the ClassicRenkoConsolidator class.
        
        
        Please use the new RenkoConsolidator if RenkoType is not Classic
        
        :param bar_size: The constant value size of each bar
        :param type: The RenkoType of the bar
        """
        ...

    def create_new_bar(self, data: QuantConnect.Data.IBaseData, current_value: float, volume: float) -> None:
        """
        Creates a new bar with the given data
        
        
        This codeEntityType is protected.
        
        :param data: The new data for the bar
        :param current_value: The new value for the bar
        :param volume: The new volume to the bar
        """
        ...

    def reset(self) -> None:
        """Resets the ClassicRenkoConsolidator"""
        ...

    def update(self, data: QuantConnect_Data_Consolidators_ClassicRenkoConsolidator_TInput) -> None:
        """
        Updates this consolidator with the specified data.
        
        :param data: The new data for the consolidator
        """
        ...

    def update_bar(self, time: typing.Union[datetime.datetime, datetime.date], current_value: float, volume: float) -> None:
        """
        Updates the current RangeBar being created with the given data.
        Additionally, if it's the case, it consolidates the current RangeBar
        
        
        This codeEntityType is protected.
        
        :param time: Time of the given data
        :param current_value: Value of the given data
        :param volume: Volume of the given data
        """
        ...


class TradeBarConsolidatorBase(typing.Generic[QuantConnect_Data_Consolidators_TradeBarConsolidatorBase_T], QuantConnect.Data.Consolidators.PeriodCountConsolidatorBase[QuantConnect_Data_Consolidators_TradeBarConsolidatorBase_T, QuantConnect.Data.Market.TradeBar], metaclass=abc.ABCMeta):
    """
    A data consolidator that can make bigger bars from any base data
    
    This type acts as the base for other consolidators that produce bars on a given time step or for a count of data.
    """

    @property
    def working_bar(self) -> QuantConnect.Data.Market.TradeBar:
        """Gets a copy of the current 'workingBar'."""
        ...

    @overload
    def __init__(self, pyfuncobj: typing.Any) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        
        This codeEntityType is protected.
        
        :param pyfuncobj: Python function object that defines the start time of a consolidated data
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta, start_time: typing.Optional[datetime.timedelta] = None) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the period
        
        
        This codeEntityType is protected.
        
        :param period: The minimum span of time before emitting a consolidated bar
        :param start_time: Optionally the bar start time anchor to use
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data
        
        
        This codeEntityType is protected.
        
        :param max_count: The number of pieces to accept before emiting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        
        This codeEntityType is protected.
        
        :param max_count: The number of pieces to accept before emiting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        
        This codeEntityType is protected.
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...


class RenkoConsolidator(typing.Generic[QuantConnect_Data_Consolidators_RenkoConsolidator_TInput], QuantConnect_Data_Consolidators_RenkoConsolidator):
    """Provides a type safe wrapper on the RenkoConsolidator class. This just allows us to define our selector functions with the real type they'll be receiving"""

    @property
    def close_on(self) -> datetime.datetime:
        """
        Time of consolidated close.
        
        
        This codeEntityType is protected.
        """
        ...

    @close_on.setter
    def close_on(self, value: datetime.datetime) -> None:
        ...

    @property
    def close_rate(self) -> float:
        """
        Value of consolidated close.
        
        
        This codeEntityType is protected.
        """
        ...

    @close_rate.setter
    def close_rate(self, value: float) -> None:
        ...

    @property
    def high_rate(self) -> float:
        """
        Value of consolidated high.
        
        
        This codeEntityType is protected.
        """
        ...

    @high_rate.setter
    def high_rate(self, value: float) -> None:
        ...

    @property
    def low_rate(self) -> float:
        """
        Value of consolidated low.
        
        
        This codeEntityType is protected.
        """
        ...

    @low_rate.setter
    def low_rate(self, value: float) -> None:
        ...

    @property
    def open_on(self) -> datetime.datetime:
        """
        Time of consolidated open.
        
        
        This codeEntityType is protected.
        """
        ...

    @open_on.setter
    def open_on(self, value: datetime.datetime) -> None:
        ...

    @property
    def open_rate(self) -> float:
        """
        Value of consolidate open.
        
        
        This codeEntityType is protected.
        """
        ...

    @open_rate.setter
    def open_rate(self, value: float) -> None:
        ...

    @property
    def bar_size(self) -> float:
        """
        Size of the consolidated bar.
        
        
        This codeEntityType is protected.
        """
        ...

    @bar_size.setter
    def bar_size(self, value: float) -> None:
        ...

    @property
    def type(self) -> QuantConnect.Data.Market.RenkoType:
        """Gets the kind of the bar"""
        ...

    @property
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def input_type(self) -> typing.Type:
        """Gets the type consumed by this consolidator"""
        ...

    @property
    def output_type(self) -> typing.Type:
        """Gets RenkoBar which is the type emitted in the IDataConsolidator.data_consolidated event."""
        ...

    @property
    def consolidated(self) -> QuantConnect.Data.IBaseData:
        """
        Gets the most recently consolidated piece of data. This will be null if this consolidator
        has not produced any data yet.
        """
        ...

    @property
    def data_consolidated(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.Market.RenkoBar], typing.Any], typing.Any]:
        """Event handler that fires when a new piece of data is produced"""
        ...

    @data_consolidated.setter
    def data_consolidated(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.Market.RenkoBar], typing.Any], typing.Any]) -> None:
        ...

    def __init__(self, bar_size: float) -> None:
        """
        Initializes a new instance of the RenkoConsolidator class using the specified bar_size.
        
        :param bar_size: The constant value size of each bar
        """
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    @staticmethod
    def get_closest_multiple(price: float, bar_size: float) -> float:
        """
        Gets the closest BarSize-Multiple to the price.
        
        :param price: Price to be rounded to the closest BarSize-Multiple
        :param bar_size: The size of the Renko bar
        :returns: The closest BarSize-Multiple to the price.
        """
        ...

    def on_data_consolidated(self, consolidated: QuantConnect.Data.Market.RenkoBar) -> None:
        """
        Event invocator for the DataConsolidated event. This should be invoked
        by derived classes when they have consolidated a new piece of data.
        
        
        This codeEntityType is protected.
        
        :param consolidated: The newly consolidated data
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    @overload
    def update(self, data: QuantConnect.Data.IBaseData) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...

    @overload
    def update(self, data: QuantConnect_Data_Consolidators_RenkoConsolidator_TInput) -> None:
        """
        Updates this consolidator with the specified data.
        
        :param data: The new data for the consolidator
        """
        ...


class WickedRenkoConsolidator(typing.Generic[QuantConnect_Data_Consolidators_WickedRenkoConsolidator_T], QuantConnect.Data.Consolidators.RenkoConsolidator[QuantConnect_Data_Consolidators_WickedRenkoConsolidator_T]):
    """
    This consolidator can transform a stream of BaseData instances into a stream of RenkoBar
    with Renko type RenkoType.WICKED.
    Provides a type safe wrapper on the WickedRenkoConsolidator class. This just allows us to define our selector functions with the real type they'll be receiving
    ///
    """

    def __init__(self, bar_size: float) -> None:
        """
        Initializes a new instance of the RenkoConsolidator class using the specified bar_size.
        
        :param bar_size: The constant value size of each bar
        """
        ...


class DataConsolidator(typing.Generic[QuantConnect_Data_Consolidators_DataConsolidator_TInput], System.Object, QuantConnect.Data.Consolidators.IDataConsolidator, metaclass=abc.ABCMeta):
    """
    Represents a type that consumes BaseData instances and fires an event with consolidated
    and/or aggregated data.
    """

    @property
    def data_consolidated(self) -> _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any], typing.Any]:
        """Event handler that fires when a new piece of data is produced"""
        ...

    @data_consolidated.setter
    def data_consolidated(self, value: _EventContainer[typing.Callable[[System.Object, QuantConnect.Data.IBaseData], typing.Any], typing.Any]) -> None:
        ...

    @property
    def consolidated(self) -> QuantConnect.Data.IBaseData:
        """
        Gets the most recently consolidated piece of data. This will be null if this consolidator
        has not produced any data yet.
        """
        ...

    @consolidated.setter
    def consolidated(self, value: QuantConnect.Data.IBaseData) -> None:
        ...

    @property
    @abc.abstractmethod
    def working_data(self) -> QuantConnect.Data.IBaseData:
        """Gets a clone of the data being currently consolidated"""
        ...

    @property
    def input_type(self) -> typing.Type:
        """Gets the type consumed by this consolidator"""
        ...

    @property
    @abc.abstractmethod
    def output_type(self) -> typing.Type:
        """Gets the type produced by this consolidator"""
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def on_data_consolidated(self, consolidated: QuantConnect.Data.IBaseData) -> None:
        """
        Event invocator for the DataConsolidated event. This should be invoked
        by derived classes when they have consolidated a new piece of data.
        
        
        This codeEntityType is protected.
        
        :param consolidated: The newly consolidated data
        """
        ...

    def reset(self) -> None:
        """Resets the consolidator"""
        ...

    def scan(self, current_local_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Scans this consolidator to see if it should emit a bar due to time passing
        
        :param current_local_time: The current time in the local time zone (same as BaseData.time)
        """
        ...

    @overload
    def update(self, data: QuantConnect.Data.IBaseData) -> None:
        """
        Updates this consolidator with the specified data
        
        :param data: The new data for the consolidator
        """
        ...

    @overload
    def update(self, data: QuantConnect_Data_Consolidators_DataConsolidator_TInput) -> None:
        """
        Updates this consolidator with the specified data. This method is
        responsible for raising the DataConsolidated event
        
        :param data: The new data for the consolidator
        """
        ...


class QuoteBarConsolidator(QuantConnect.Data.Consolidators.PeriodCountConsolidatorBase[QuantConnect.Data.Market.QuoteBar, QuantConnect.Data.Market.QuoteBar]):
    """Consolidates QuoteBars into larger QuoteBars"""

    @overload
    def __init__(self, pyfuncobj: typing.Any) -> None:
        """
        Creates a consolidator to produce a new 'QuoteBar' representing the last count pieces of data or the period, whichever comes first
        
        :param pyfuncobj: Python function object that defines the start time of a consolidated data
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta, start_time: typing.Optional[datetime.timedelta] = None) -> None:
        """
        Initializes a new instance of the QuoteBarConsolidator class
        
        :param period: The minimum span of time before emitting a consolidated bar
        :param start_time: Optionally the bar start time anchor to use
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Initializes a new instance of the QuoteBarConsolidator class
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Initializes a new instance of the QuoteBarConsolidator class
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Creates a consolidator to produce a new 'QuoteBar' representing the last count pieces of data or the period, whichever comes first
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.QuoteBar, data: QuantConnect.Data.Market.QuoteBar) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new consolidated bar
        :param data: The new data
        """
        ...


class DynamicDataConsolidator(QuantConnect.Data.Consolidators.TradeBarConsolidatorBase[QuantConnect.Data.DynamicData]):
    """
    A data csolidator that can make trade bars from DynamicData derived types. This is useful for
    aggregating Quandl and other highly flexible dynamic custom data types.
    """

    @overload
    def __init__(self, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the period.
        
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data.
        
        :param max_count: The number of pieces to accept before emiting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first.
        
        :param max_count: The number of pieces to accept before emiting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first.
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.TradeBar, data: QuantConnect.Data.DynamicData) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new trade bar
        :param data: The new data
        """
        ...


class TickConsolidator(QuantConnect.Data.Consolidators.TradeBarConsolidatorBase[QuantConnect.Data.Market.Tick]):
    """
    A data consolidator that can make bigger bars from ticks over a given
    time span or a count of pieces of data.
    """

    @overload
    def __init__(self, pyfuncobj: typing.Any) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        :param pyfuncobj: Python function object that defines the start time of a consolidated data
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the period
        
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'TradeBar' representing the last count pieces of data or the period, whichever comes first
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Initializes a new instance of the TickQuoteBarConsolidator class
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.TradeBar, data: QuantConnect.Data.Market.Tick) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building
        :param data: The new data
        """
        ...

    def should_process(self, data: QuantConnect.Data.Market.Tick) -> bool:
        """
        Determines whether or not the specified data should be processed
        
        
        This codeEntityType is protected.
        
        :param data: The data to check
        :returns: True if the consolidator should process this data, false otherwise.
        """
        ...


class TickQuoteBarConsolidator(QuantConnect.Data.Consolidators.PeriodCountConsolidatorBase[QuantConnect.Data.Market.Tick, QuantConnect.Data.Market.QuoteBar]):
    """Consolidates ticks into quote bars. This consolidator ignores trade ticks"""

    @overload
    def __init__(self, pyfuncobj: typing.Any) -> None:
        """
        Initializes a new instance of the TickQuoteBarConsolidator class
        
        :param pyfuncobj: Python function object that defines the start time of a consolidated data
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta, start_time: typing.Optional[datetime.timedelta] = None) -> None:
        """
        Initializes a new instance of the TickQuoteBarConsolidator class
        
        :param period: The minimum span of time before emitting a consolidated bar
        :param start_time: Optionally the bar start time anchor to use
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Initializes a new instance of the TickQuoteBarConsolidator class
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Initializes a new instance of the TickQuoteBarConsolidator class
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Initializes a new instance of the TickQuoteBarConsolidator class
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.QuoteBar, data: QuantConnect.Data.Market.Tick) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new consolidated bar
        :param data: The new data
        """
        ...

    def should_process(self, data: QuantConnect.Data.Market.Tick) -> bool:
        """
        Determines whether or not the specified data should be processed
        
        
        This codeEntityType is protected.
        
        :param data: The data to check
        :returns: True if the consolidator should process this data, false otherwise.
        """
        ...


class OpenInterestConsolidator(QuantConnect.Data.Consolidators.PeriodCountConsolidatorBase[QuantConnect.Data.Market.Tick, QuantConnect.Data.Market.OpenInterest]):
    """Type capable of consolidating open interest"""

    @overload
    def __init__(self, pyfuncobj: typing.Any) -> None:
        """
        Creates a consolidator to produce a new 'OpenInterest'
        
        :param pyfuncobj: Python function object that defines the start time of a consolidated data
        """
        ...

    @overload
    def __init__(self, period: datetime.timedelta, start_time: typing.Optional[datetime.timedelta] = None) -> None:
        """
        Creates a consolidator to produce a new 'OpenInterest' representing the period
        
        :param period: The minimum span of time before emitting a consolidated bar
        :param start_time: Optionally the bar start time anchor to use
        """
        ...

    @overload
    def __init__(self, max_count: int) -> None:
        """
        Creates a consolidator to produce a new 'OpenInterest' representing the last count pieces of data
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, max_count: int, period: datetime.timedelta) -> None:
        """
        Creates a consolidator to produce a new 'OpenInterest' representing the last count pieces of data or the period, whichever comes first
        
        :param max_count: The number of pieces to accept before emitting a consolidated bar
        :param period: The minimum span of time before emitting a consolidated bar
        """
        ...

    @overload
    def __init__(self, func: typing.Callable[[datetime.datetime], QuantConnect.Data.Consolidators.CalendarInfo]) -> None:
        """
        Creates a consolidator to produce a new 'OpenInterest'
        
        :param func: Func that defines the start time of a consolidated data
        """
        ...

    def aggregate_bar(self, working_bar: QuantConnect.Data.Market.OpenInterest, data: QuantConnect.Data.Market.Tick) -> None:
        """
        Aggregates the new 'data' into the 'working_bar'. The 'working_bar' will be
        null following the event firing
        
        
        This codeEntityType is protected.
        
        :param working_bar: The bar we're building, null if the event was just fired and we're starting a new OI bar
        :param data: The new data
        """
        ...

    @staticmethod
    def from_resolution(resolution: QuantConnect.Resolution) -> QuantConnect.Data.Consolidators.OpenInterestConsolidator:
        """
        Create a new OpenInterestConsolidator for the desired resolution
        
        :param resolution: The resolution desired
        :returns: A consolidator that produces data on the resolution interval.
        """
        ...

    def should_process(self, data: QuantConnect.Data.Market.Tick) -> bool:
        """
        Determines whether or not the specified data should be processed
        
        
        This codeEntityType is protected.
        
        :param data: The data to check
        :returns: True if the consolidator should process this data, false otherwise.
        """
        ...

    def update(self, data: QuantConnect.Data.Market.Tick) -> None:
        """
        Updates this consolidator with the specified data. This method is
        responsible for raising the DataConsolidated event.
        It will check for date or hour change and force consolidation if needed.
        
        :param data: The new data for the consolidator
        """
        ...


class _EventContainer(typing.Generic[QuantConnect_Data_Consolidators__EventContainer_Callable, QuantConnect_Data_Consolidators__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_Data_Consolidators__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_Data_Consolidators__EventContainer_Callable) -> typing.Self:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_Data_Consolidators__EventContainer_Callable) -> typing.Self:
        """Unregisters an event handler."""
        ...





from typing import overload
from enum import IntEnum
import typing

import QuantConnect
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.DataFeeds.Queues
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.Collections.Generic


class LiveDataQueue(System.Object, QuantConnect.Interfaces.IDataQueueHandler):
    """Live Data Queue is the cut out implementation of how to bind a custom live data source"""

    @property
    def is_connected(self) -> bool:
        """Returns whether the data provider is connected"""
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def set_job(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Sets the job we're subscribing for
        
        :param job: Job we're subscribing for
        """
        ...

    def subscribe(self, data_config: QuantConnect.Data.SubscriptionDataConfig, new_data_available_handler: typing.Callable[[System.Object, System.EventArgs], typing.Any]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """Desktop/Local doesn't support live data from this handler"""
        ...

    def unsubscribe(self, data_config: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """Desktop/Local doesn't support live data from this handler"""
        ...


class FakeDataQueue(System.Object, QuantConnect.Interfaces.IDataQueueHandler, QuantConnect.Interfaces.IDataQueueUniverseProvider):
    """This is an implementation of IDataQueueHandler used for testing. FakeHistoryProvider"""

    @property
    def time_provider(self) -> QuantConnect.ITimeProvider:
        """
        Continuous UTC time provider
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def is_connected(self) -> bool:
        """Returns whether the data provider is connected"""
        ...

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of the FakeDataQueue class to randomly emit data for each symbol"""
        ...

    @overload
    def __init__(self, data_aggregator: QuantConnect.Data.IDataAggregator, data_points_per_second_per_symbol: int = 500000) -> None:
        """Initializes a new instance of the FakeDataQueue class to randomly emit data for each symbol"""
        ...

    def can_perform_selection(self) -> bool:
        """Checks if the FakeDataQueue can perform selection"""
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def lookup_symbols(self, symbol: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract, QuantConnect.Securities.Security], include_expired: bool, security_currency: str = None) -> typing.Iterable[QuantConnect.Symbol]:
        """
        Method returns a collection of Symbols that are available at the data source.
        
        :param symbol: Symbol to lookup
        :param include_expired: Include expired contracts
        :param security_currency: Expected security currency(if any)
        :returns: Enumerable of Symbols, that are associated with the provided Symbol.
        """
        ...

    def set_job(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Sets the job we're subscribing for
        
        :param job: Job we're subscribing for
        """
        ...

    def subscribe(self, data_config: QuantConnect.Data.SubscriptionDataConfig, new_data_available_handler: typing.Callable[[System.Object, System.EventArgs], typing.Any]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param data_config: defines the parameters to subscribe to a data feed
        :param new_data_available_handler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    def unsubscribe(self, data_config: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param data_config: Subscription config to be removed
        """
        ...





from typing import overload
from enum import IntEnum
import datetime
import typing

import QuantConnect
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.DataFeeds.DataDownloader
import QuantConnect.Securities
import System


class CanonicalDataDownloaderDecorator(System.Object, QuantConnect.IDataDownloader):
    """
    Decorates an IDataDownloader to support canonical symbols by automatically
    resolving their option or future contract chains and downloading data for each constituent contract.
    """

    def __init__(self, data_downloader: QuantConnect.IDataDownloader, data_provider: QuantConnect.Interfaces.IDataProvider, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider) -> None:
        """
        Initializes a new instance of the CanonicalDataDownloaderDecorator class.
        
        :param data_downloader: The underlying data downloader to decorate with canonical symbol support.
        :param data_provider: The data provider used for initializing chain providers.
        :param map_file_provider: The map file provider used for initializing chain providers.
        :param factor_file_provider: The factor file provider used for initializing chain providers.
        """
        ...

    def get(self, data_downloader_get_parameters: QuantConnect.DataDownloaderGetParameters) -> typing.Iterable[QuantConnect.Data.BaseData]:
        """
        Get historical data enumerable for a single symbol, type and resolution given this start and end time (in UTC).
        For canonical symbols, automatically resolves and downloads data for all underlying contracts.
        
        :param data_downloader_get_parameters: model class for passing in parameters for historical data
        :returns: Enumerable of base data for this symbol.
        """
        ...

    @staticmethod
    def try_adjust_date_range_for_contract(contract: typing.Union[QuantConnect.Symbol, str, QuantConnect.Data.Market.BaseContract, QuantConnect.Securities.Security], original_start_date_utc: typing.Union[datetime.datetime, datetime.date], original_end_date_utc: typing.Union[datetime.datetime, datetime.date], adjusted_start_date_utc: typing.Optional[typing.Union[datetime.datetime, datetime.date]], adjusted_end_date_utc: typing.Optional[typing.Union[datetime.datetime, datetime.date]]) -> typing.Tuple[bool, typing.Union[datetime.datetime, datetime.date], typing.Union[datetime.datetime, datetime.date]]:
        """
        Tries to adjust the date range for a given contract based on its security type and expiry date.
        The start date is clamped to a minimum look-back period and the end date is clamped to the contract expiry date.
        Returns false if the minimum look-back period exceeds the requested end date, meaning no valid range exists.
        
        :param contract: The contract symbol containing the security type and expiry date.
        :param original_start_date_utc: The requested start date in UTC.
        :param original_end_date_utc: The requested end date in UTC.
        :param adjusted_start_date_utc: The adjusted start date in UTC, or default if no valid range exists.
        :param adjusted_end_date_utc: The adjusted end date in UTC, or default if no valid range exists.
        :returns: True if a valid adjusted date range was found, false otherwise.
        """
        ...


class DataDownloaderSelector(System.Object, System.IDisposable):
    """Selects the appropriate data downloader based on the data type."""

    def __init__(self, base_data_downloader: QuantConnect.IDataDownloader, map_file_provider: QuantConnect.Interfaces.IMapFileProvider, data_provider: QuantConnect.Interfaces.IDataProvider, factor_file_provider: QuantConnect.Interfaces.IFactorFileProvider = None) -> None:
        """
        Initializes a new instance of the DataDownloaderSelector class.
        
        :param base_data_downloader: The base data downloader instance.
        :param map_file_provider: The map file provider used for initializing chain providers.
        :param data_provider: The data provider used for initializing chain providers.
        :param factor_file_provider: The factor file provider used for initializing chain providers.
        """
        ...

    def dispose(self) -> None:
        """Disposes the base downloader and the decorator if it was initialized."""
        ...

    def get_data_downloader(self, data_type: typing.Type) -> QuantConnect.IDataDownloader:
        """
        Returns the appropriate downloader for the given data type.
        
        :param data_type: The type of data to download.
        :returns: The base downloader for common lean data types, otherwise the canonical decorator.
        """
        ...





from typing import overload
from enum import IntEnum
import abc
import datetime
import typing

import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Orders.Fills
import QuantConnect.Python
import QuantConnect.Securities
import System
import System.Collections.Generic


class Fill(System.Object, typing.Iterable[QuantConnect.Orders.OrderEvent]):
    """Defines a possible result for IfillModel.fill for a single order"""

    @overload
    def __init__(self, order_events: typing.List[QuantConnect.Orders.OrderEvent]) -> None:
        """
        Creates a new Fill instance
        
        :param order_events: The fill order events
        """
        ...

    @overload
    def __init__(self, order_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        Creates a new Fill instance
        
        :param order_event: The fill order event
        """
        ...

    def __iter__(self) -> typing.Iterator[QuantConnect.Orders.OrderEvent]:
        ...

    def get_enumerator(self) -> System.Collections.Generic.IEnumerator[QuantConnect.Orders.OrderEvent]:
        """Returns the order events enumerator"""
        ...


class FillModelParameters(System.Object):
    """Defines the parameters for the IFillModel method"""

    @property
    def security(self) -> QuantConnect.Securities.Security:
        """Gets the security"""
        ...

    @property
    def order(self) -> QuantConnect.Orders.Order:
        """Gets the order"""
        ...

    @property
    def config_provider(self) -> QuantConnect.Interfaces.ISubscriptionDataConfigProvider:
        """Gets the SubscriptionDataConfig provider"""
        ...

    @property
    def stale_price_time_span(self) -> datetime.timedelta:
        """Gets the minimum time span elapsed to consider a market fill price as stale (defaults to one hour)"""
        ...

    @property
    def securities_for_orders(self) -> System.Collections.Generic.Dictionary[QuantConnect.Orders.Order, QuantConnect.Securities.Security]:
        """Gets the collection of securities by order"""
        ...

    @property
    def on_order_updated(self) -> typing.Callable[[QuantConnect.Orders.Order], typing.Any]:
        """Callback to notify when an order is updated by the fill model"""
        ...

    def __init__(self, security: QuantConnect.Securities.Security, order: QuantConnect.Orders.Order, config_provider: QuantConnect.Interfaces.ISubscriptionDataConfigProvider, stale_price_time_span: datetime.timedelta, securities_for_orders: System.Collections.Generic.Dictionary[QuantConnect.Orders.Order, QuantConnect.Securities.Security], on_order_updated: typing.Callable[[QuantConnect.Orders.Order], typing.Any] = None) -> None:
        """
        Creates a new instance
        
        :param security: Security asset we're filling
        :param order: Order packet to model
        :param config_provider: The ISubscriptionDataConfigProvider to use
        :param stale_price_time_span: The minimum time span elapsed to consider a fill price as stale
        :param securities_for_orders: Collection of securities for each order
        """
        ...


class IFillModel(metaclass=abc.ABCMeta):
    """Represents a model that simulates order fill events"""

    def fill(self, parameters: QuantConnect.Orders.Fills.FillModelParameters) -> QuantConnect.Orders.Fills.Fill:
        """
        Return an order event with the fill details
        
        :param parameters: A FillModelParameters object containing the security and order
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...


class Prices(System.Object):
    """Prices class used by IFillModels"""

    @property
    def end_time(self) -> datetime.datetime:
        """End time for these prices"""
        ...

    @property
    def current(self) -> float:
        """Current price"""
        ...

    @property
    def open(self) -> float:
        """Open price"""
        ...

    @property
    def high(self) -> float:
        """High price"""
        ...

    @property
    def low(self) -> float:
        """Low price"""
        ...

    @property
    def close(self) -> float:
        """Closing price"""
        ...

    @overload
    def __init__(self, bar: QuantConnect.Data.Market.IBaseDataBar) -> None:
        """
        Create an instance of Prices class with a data bar
        
        :param bar: Data bar to use for prices
        """
        ...

    @overload
    def __init__(self, end_time: typing.Union[datetime.datetime, datetime.date], bar: QuantConnect.Data.Market.IBar) -> None:
        """
        Create an instance of Prices class with a data bar and end time
        
        :param end_time: The end time for these prices
        :param bar: Data bar to use for prices
        """
        ...

    @overload
    def __init__(self, end_time: typing.Union[datetime.datetime, datetime.date], current: float, open: float, high: float, low: float, close: float) -> None:
        """
        Create a instance of the Prices class with specific values for all prices
        
        :param end_time: The end time for these prices
        :param current: Current price
        :param open: Open price
        :param high: High price
        :param low: Low price
        :param close: Close price
        """
        ...


class FillModel(System.Object, QuantConnect.Orders.Fills.IFillModel):
    """Provides a base class for all fill models"""

    @property
    def parameters(self) -> QuantConnect.Orders.Fills.FillModelParameters:
        """
        The parameters instance to be used by the different XxxxFill() implementations
        
        
        This codeEntityType is protected.
        """
        ...

    @parameters.setter
    def parameters(self, value: QuantConnect.Orders.Fills.FillModelParameters) -> None:
        ...

    @property
    def python_wrapper(self) -> QuantConnect.Python.FillModelPythonWrapper:
        """
        This is required due to a limitation in PythonNet to resolved overriden methods.
        When Python calls a C# method that calls a method that's overriden in python it won't
        run the python implementation unless the call is performed through python too.
        
        
        This codeEntityType is protected.
        """
        ...

    @python_wrapper.setter
    def python_wrapper(self, value: QuantConnect.Python.FillModelPythonWrapper) -> None:
        ...

    def combo_leg_limit_fill(self, order: QuantConnect.Orders.Order, parameters: QuantConnect.Orders.Fills.FillModelParameters) -> typing.List[QuantConnect.Orders.OrderEvent]:
        """
        Default combo limit fill model for the base security class. Fills at the limit price for each leg
        
        :param order: Order to fill
        :param parameters: Fill parameters for the order
        :returns: Order fill information detailing the average price and quantity filled for each leg. If any of the fills fails, none of the orders will be filled and the returned list will be empty.
        """
        ...

    def combo_limit_fill(self, order: QuantConnect.Orders.Order, parameters: QuantConnect.Orders.Fills.FillModelParameters) -> typing.List[QuantConnect.Orders.OrderEvent]:
        """
        Default combo limit fill model for the base security class. Fills at the sum of prices for the assets of every leg.
        
        :param order: Order to fill
        :param parameters: Fill parameters for the order
        :returns: Order fill information detailing the average price and quantity filled for each leg. If any of the fills fails, none of the orders will be filled and the returned list will be empty.
        """
        ...

    def combo_market_fill(self, order: QuantConnect.Orders.Order, parameters: QuantConnect.Orders.Fills.FillModelParameters) -> typing.List[QuantConnect.Orders.OrderEvent]:
        """
        Default combo market fill model for the base security class. Fills at the last traded price for each leg.
        
        :param order: Order to fill
        :param parameters: Fill parameters for the order
        :returns: Order fill information detailing the average price and quantity filled for each leg. If any of the fills fails, none of the orders will be filled and the returned list will be empty.
        """
        ...

    def fill(self, parameters: QuantConnect.Orders.Fills.FillModelParameters) -> QuantConnect.Orders.Fills.Fill:
        """
        Return an order event with the fill details
        
        :param parameters: A FillModelParameters object containing the security and order
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def get_prices(self, asset: QuantConnect.Securities.Security, direction: QuantConnect.Orders.OrderDirection) -> QuantConnect.Orders.Fills.Prices:
        """
        Get the minimum and maximum price for this security in the last bar:
        
        
        This codeEntityType is protected.
        
        :param asset: Security asset we're checking
        :param direction: The order direction, decides whether to pick bid or ask
        """
        ...

    def get_prices_checking_python_wrapper(self, asset: QuantConnect.Securities.Security, direction: QuantConnect.Orders.OrderDirection) -> QuantConnect.Orders.Fills.Prices:
        """
        This is required due to a limitation in PythonNet to resolved
        overriden methods. get_prices
        
        This codeEntityType is protected.
        """
        ...

    def get_subscribed_types(self, asset: QuantConnect.Securities.Security) -> System.Collections.Generic.HashSet[typing.Type]:
        """
        Get data types the Security is subscribed to
        
        
        This codeEntityType is protected.
        
        :param asset: Security which has subscribed data types
        """
        ...

    def is_exchange_open(self, asset: QuantConnect.Securities.Security, is_extended_market_hours: bool) -> bool:
        """
        Determines if the exchange is open using the current time of the asset
        
        
        This codeEntityType is protected.
        """
        ...

    def limit_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.LimitOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default limit order fill model in the base security class.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def limit_if_touched_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.LimitIfTouchedOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default limit if touched fill model implementation in base class security. (Limit If Touched Order Type)
        
        :param asset: 
        :param order: 
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def market_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.MarketOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default market fill model for the base security class. Fills at the last traded price.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def market_on_close_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.MarketOnCloseOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Market on Close Fill Model. Return an order event with the fill details
        
        :param asset: Asset we're trading with this order
        :param order: Order to be filled
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def market_on_open_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.MarketOnOpenOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Market on Open Fill Model. Return an order event with the fill details
        
        :param asset: Asset we're trading with this order
        :param order: Order to be filled
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def set_python_wrapper(self, python_wrapper: QuantConnect.Python.FillModelPythonWrapper) -> None:
        """Used to set the FillModelPythonWrapper instance if any"""
        ...

    def stop_limit_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.StopLimitOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default stop limit fill model implementation in base class security. (Stop Limit Order Type)
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def stop_market_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.StopMarketOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default stop fill model implementation in base class security. (Stop Market Order Type)
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def trailing_stop_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.TrailingStopOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default trailing stop fill model implementation in base class security. (Trailing Stop Order Type)
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...


class ImmediateFillModel(QuantConnect.Orders.Fills.FillModel):
    """Represents the default fill model used to simulate order fills"""


class FutureFillModel(QuantConnect.Orders.Fills.ImmediateFillModel):
    """Represents the fill model used to simulate order fills for futures"""

    def market_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.MarketOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default market fill model for the base security class. Fills at the last traded price.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def stop_market_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.StopMarketOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Stop fill model implementation for Future.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...


class FutureOptionFillModel(QuantConnect.Orders.Fills.FutureFillModel):
    """Represents the default fill model used to simulate order fills for future options"""


class LatestPriceFillModel(QuantConnect.Orders.Fills.ImmediateFillModel):
    """
    This fill model is provided for cases where the trade/quote distinction should be
    ignored and the fill price should be determined from the latest pricing information.
    """

    def get_prices(self, asset: QuantConnect.Securities.Security, direction: QuantConnect.Orders.OrderDirection) -> QuantConnect.Orders.Fills.Prices:
        """
        Get the minimum and maximum price for this security in the last bar
        Ignore the Trade/Quote distinction - fill with the latest pricing information
        
        
        This codeEntityType is protected.
        
        :param asset: Security asset we're checking
        :param direction: The order direction, decides whether to pick bid or ask
        """
        ...


class EquityFillModel(QuantConnect.Orders.Fills.FillModel):
    """Represents the fill model used to simulate order fills for equities"""

    def get_prices(self, asset: QuantConnect.Securities.Security, direction: QuantConnect.Orders.OrderDirection) -> QuantConnect.Orders.Fills.Prices:
        """
        Get the minimum and maximum price for this security in the last bar:
        
        
        This codeEntityType is protected.
        
        :param asset: Security asset we're checking
        :param direction: The order direction, decides whether to pick bid or ask
        """
        ...

    def get_prices_checking_python_wrapper(self, asset: QuantConnect.Securities.Security, direction: QuantConnect.Orders.OrderDirection) -> QuantConnect.Orders.Fills.Prices:
        """
        This is required due to a limitation in PythonNet to resolved
        overriden methods. get_prices
        
        This codeEntityType is protected.
        """
        ...

    def get_subscribed_types(self, asset: QuantConnect.Securities.Security) -> System.Collections.Generic.HashSet[typing.Type]:
        """
        Get data types the Security is subscribed to
        
        
        This codeEntityType is protected.
        
        :param asset: Security which has subscribed data types
        """
        ...

    def limit_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.LimitOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Limit fill model implementation for Equity.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def limit_if_touched_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.LimitIfTouchedOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default limit if touched fill model implementation in base class security.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def market_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.MarketOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default market fill model for the base security class. Fills at the last traded price.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def market_on_close_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.MarketOnCloseOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Market on Close Fill Model. Return an order event with the fill details
        
        :param asset: Asset we're trading with this order
        :param order: Order to be filled
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def market_on_open_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.MarketOnOpenOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Market on Open Fill Model. Return an order event with the fill details
        
        :param asset: Asset we're trading with this order
        :param order: Order to be filled
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def stop_limit_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.StopLimitOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Default stop limit fill model implementation in base class security. (Stop Limit Order Type)
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...

    def stop_market_fill(self, asset: QuantConnect.Securities.Security, order: QuantConnect.Orders.StopMarketOrder) -> QuantConnect.Orders.OrderEvent:
        """
        Stop fill model implementation for Equity.
        
        :param asset: Security asset we're filling
        :param order: Order packet to model
        :returns: Order fill information detailing the average price and quantity filled.
        """
        ...





from typing import overload
from enum import IntEnum
import typing

import System
import System.Numerics.Colors

System_Numerics_Colors_Argb = typing.Any
System_Numerics_Colors_Rgba = typing.Any

System_Numerics_Colors_Argb_T = typing.TypeVar("System_Numerics_Colors_Argb_T")
System_Numerics_Colors_Rgba_T = typing.TypeVar("System_Numerics_Colors_Rgba_T")


class Argb(typing.Generic[System_Numerics_Colors_Argb_T], System.IEquatable[System_Numerics_Colors_Argb]):
    """This class has no documentation."""

    @property
    def a(self) -> System_Numerics_Colors_Argb_T:
        ...

    @property
    def r(self) -> System_Numerics_Colors_Argb_T:
        ...

    @property
    def g(self) -> System_Numerics_Colors_Argb_T:
        ...

    @property
    def b(self) -> System_Numerics_Colors_Argb_T:
        ...

    @overload
    def __init__(self, a: System_Numerics_Colors_Argb_T, r: System_Numerics_Colors_Argb_T, g: System_Numerics_Colors_Argb_T, b: System_Numerics_Colors_Argb_T) -> None:
        ...

    @overload
    def __init__(self, values: System.ReadOnlySpan[System_Numerics_Colors_Argb_T]) -> None:
        ...

    def copy_to(self, destination: System.Span[System_Numerics_Colors_Argb_T]) -> None:
        ...

    @staticmethod
    def create_big_endian(color: int) -> System.Numerics.Colors.Argb[int]:
        ...

    @staticmethod
    def create_little_endian(color: int) -> System.Numerics.Colors.Argb[int]:
        ...

    @overload
    def equals(self, obj: typing.Any) -> bool:
        ...

    @overload
    def equals(self, other: System.Numerics.Colors.Argb[System_Numerics_Colors_Argb_T]) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    def to_rgba(self) -> System.Numerics.Colors.Rgba[System_Numerics_Colors_Argb_T]:
        ...

    def to_string(self) -> str:
        ...

    @staticmethod
    def to_u_int_32_big_endian(color: System.Numerics.Colors.Argb[int]) -> int:
        ...

    @staticmethod
    def to_u_int_32_little_endian(color: System.Numerics.Colors.Argb[int]) -> int:
        ...


class Rgba(typing.Generic[System_Numerics_Colors_Rgba_T], System.IEquatable[System_Numerics_Colors_Rgba]):
    """This class has no documentation."""

    @property
    def r(self) -> System_Numerics_Colors_Rgba_T:
        ...

    @property
    def g(self) -> System_Numerics_Colors_Rgba_T:
        ...

    @property
    def b(self) -> System_Numerics_Colors_Rgba_T:
        ...

    @property
    def a(self) -> System_Numerics_Colors_Rgba_T:
        ...

    @overload
    def __init__(self, r: System_Numerics_Colors_Rgba_T, g: System_Numerics_Colors_Rgba_T, b: System_Numerics_Colors_Rgba_T, a: System_Numerics_Colors_Rgba_T) -> None:
        ...

    @overload
    def __init__(self, values: System.ReadOnlySpan[System_Numerics_Colors_Rgba_T]) -> None:
        ...

    def copy_to(self, destination: System.Span[System_Numerics_Colors_Rgba_T]) -> None:
        ...

    @staticmethod
    def create_big_endian(color: int) -> System.Numerics.Colors.Rgba[int]:
        ...

    @staticmethod
    def create_little_endian(color: int) -> System.Numerics.Colors.Rgba[int]:
        ...

    @overload
    def equals(self, obj: typing.Any) -> bool:
        ...

    @overload
    def equals(self, other: System.Numerics.Colors.Rgba[System_Numerics_Colors_Rgba_T]) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    def to_argb(self) -> System.Numerics.Colors.Argb[System_Numerics_Colors_Rgba_T]:
        ...

    def to_string(self) -> str:
        ...

    @staticmethod
    def to_u_int_32_big_endian(color: System.Numerics.Colors.Rgba[int]) -> int:
        ...

    @staticmethod
    def to_u_int_32_little_endian(color: System.Numerics.Colors.Rgba[int]) -> int:
        ...


